/*
 * DinX window<->server messages
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is DinX.
 * 
 * The Initial Developer of the Original Code is Ben Williamson.
 * Portions created by Ben Williamson are Copyright (C) 1999
 * Ben Williamson. All Rights Reserved.
 * 
 * Contributor(s):	Ben Williamson <benw@pobox.com>
 * 
 * Alternatively, the contents of this file may be used under the terms
 * of the GNU General Public license (the  "[GPL] License"), in which case the
 * provisions of [GPL] License are applicable instead of those
 * above.  If you wish to allow use of your version of this file only
 * under the terms of the [GPL] License and not to allow others to use
 * your version of this file under the MPL, indicate your decision by
 * deleting  the provisions above and replace  them with the notice and
 * other provisions required by the [GPL] License.  If you do not delete
 * the provisions above, a recipient may use your version of this file
 * under either the MPL or the [GPL] License.
 */

#ifndef _DINX_MESSAGES_H_
#define _DINX_MESSAGES_H_

#include <linux/dinx.h>

enum dmessage_type {
	DMESSAGE_NONE = 0,
	
	/* from windows: */
	DMESSAGE_REQ_BOUNDS,
	DMESSAGE_REQ_MAP,
	DMESSAGE_REQ_UNMAP,
	DMESSAGE_REQ_RAISE,
	DMESSAGE_REQ_LOWER,
	
	/* from server: */
	DMESSAGE_TREE_INFO,
	DMESSAGE_MOUSE,
	DMESSAGE_KEY
};

struct dmessage_req_bounds {
	enum dmessage_type	type;	/* DMESSAGE_SET_BOUNDS */
	drect			bounds;
	dvect			origin;
};

struct dmessage_mouse {
	enum dmessage_type	type;	/* DMESSAGE_MOUSE */
	dvect			pos;
	int			buttons;
};

struct dmessage_key {
	enum dmessage_type	type;	/* DMESSAGE_KEY */
	int			key;
};

struct dmessage_tree_info {
	enum dmessage_type	type;	/* DMESSAGE_TREE_INFO */
	drect			bounds;
	int			bpp;
};

union dmessage {
	enum dmessage_type		type;
	struct dmessage_req_bounds	req_bounds;
	struct dmessage_mouse		mouse;
	struct dmessage_key		key;
	struct dmessage_tree_info	tree_info;
	char				pad[DINX_MESSAGE_SIZE];
};

#endif	/* _DINX_MESSAGES_H_ */
