/*
 * DinX server framebuffer handling
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is DinX.
 * 
 * The Initial Developer of the Original Code is Ben Williamson.
 * Portions created by Ben Williamson are Copyright (C) 1999
 * Ben Williamson. All Rights Reserved.
 * 
 * Contributor(s):	Ben Williamson <benw@pobox.com>
 * 
 * Alternatively, the contents of this file may be used under the terms
 * of the GNU General Public license (the  "[GPL] License"), in which case the
 * provisions of [GPL] License are applicable instead of those
 * above.  If you wish to allow use of your version of this file only
 * under the terms of the [GPL] License and not to allow others to use
 * your version of this file under the MPL, indicate your decision by
 * deleting  the provisions above and replace  them with the notice and
 * other provisions required by the [GPL] License.  If you do not delete
 * the provisions above, a recipient may use your version of this file
 * under either the MPL or the [GPL] License.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/sysmacros.h>
#include <sys/ioctl.h>
#include <fcntl.h>

#include <linux/fb.h>

#include "fb.h"

const char *fb_filename = "/dev/fb";

int fb_number;

struct fb_fix_screeninfo fb_fix;
struct fb_var_screeninfo fb_var;
	

int fb_fd, my_fb;

int fb_init(const char *filename)
{
	struct stat st;
	struct fb_cmap cmap;
	unsigned short red[256], green[256], blue[256];
	int r, g, b, c;
	char *env;
	
	env = getenv("FRAMEBUFFER");
	if (env) {
		fb_filename = env;
	}
	if (filename) {
		fb_filename = filename;
	}

	if (stat(fb_filename, &st)) {
		perror(fb_filename);
		return -1;
	}
	if (!S_ISCHR(st.st_mode)) {
		fprintf(stderr, "%s: not a character special device\n",
				fb_filename);
		return -1;
	}
	if (major(st.st_rdev) != FB_MAJOR) {
		fprintf(stderr, "%s: major number does not match fb.h\n",
				fb_filename);
		return -1;
	}
	fb_number = minor(st.st_rdev) >> FB_MODES_SHIFT;

	fb_fd = open(fb_filename, O_RDWR);
	if (fb_fd < 0) {
		perror(fb_filename);
		return -1;
	}

	if (ioctl(fb_fd, FBIOGET_FSCREENINFO, &fb_fix)) goto err_out;
	if (ioctl(fb_fd, FBIOGET_VSCREENINFO, &fb_var)) goto err_out;

	c = 0;
	for (r = 0; r < 6; r++) {
		for (g = 0; g < 6; g++) {
			for (b = 0; b < 6; b++) {
				red[c]   = (r * 255 / 5) << 8;
				green[c] = (g * 255 / 5) << 8;
				blue[c]  = (b * 255 / 5) << 8;
				c++;
			}
		}
	}
	red[255] = green[255] = blue[255] = 0xffff;
	cmap.start = 0;
	cmap.len = 256;
	cmap.red = red;
	cmap.green = green;
	cmap.blue = blue;
	cmap.transp = NULL;
	if (ioctl(fb_fd, FBIOPUTCMAP, &cmap)) goto err_out;
	close(fb_fd);
	return 0;
err_out:
	perror("fb_init");
	return -1;
}

