/*
 * DinX cursor window - temporary, until we have a real mouse pointer.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is DinX.
 * 
 * The Initial Developer of the Original Code is Ben Williamson.
 * Portions created by Ben Williamson are Copyright (C) 1999
 * Ben Williamson. All Rights Reserved.
 * 
 * Contributor(s):	Ben Williamson <benw@pobox.com>
 * 
 * Alternatively, the contents of this file may be used under the terms
 * of the GNU General Public license (the  "[GPL] License"), in which case the
 * provisions of [GPL] License are applicable instead of those
 * above.  If you wish to allow use of your version of this file only
 * under the terms of the [GPL] License and not to allow others to use
 * your version of this file under the MPL, indicate your decision by
 * deleting  the provisions above and replace  them with the notice and
 * other provisions required by the [GPL] License.  If you do not delete
 * the provisions above, a recipient may use your version of this file
 * under either the MPL or the [GPL] License.
 */

#include <stdio.h>

#include <dinx/access.h>

struct cursor_win {
	int fd;
	unsigned long col;
	dvect pos;
	drect bounds;
};

static void
cursor_redraw(void *data, const drect *rect, int more)
{
	struct cursor_win *win = (struct cursor_win *)data;

	dinx_fill_rect(win->fd, rect, win->col);
}

static void
cursor_mouse(void *data, const dvect *pos, int buttons)
{
	struct cursor_win *win = (struct cursor_win *)data;
	
	dinx_req_bounds(win->fd, &win->bounds, pos);
	dinx_req_raise(win->fd);
}

struct devent_ops cursor_ops = {
	NULL,		/* tree_info */
	NULL,		/* message */
	cursor_redraw,
	cursor_mouse,
	NULL		/* key */
};

int main(void)
{
	struct cursor_win win;
	dvect origin;

	win.fd = dinx_open(NULL);
	if (win.fd < 0) {
		perror("dinx_open");
		exit(1);
	}
	win.col = 0xffffffff;
	win.bounds.x1 = -5;
	win.bounds.y1 = -5;
	win.bounds.x2 = 5;
	win.bounds.y2 = 5;
	origin.x = -10;
	origin.y = -10;
	dinx_req_bounds(win.fd, &win.bounds, &origin);
	dinx_req_map(win.fd);

	while (1) {
		dinx_read_event(win.fd, &cursor_ops, &win);
	}
	return 0;
}

