/*
 * DinX screenshot grabber utility
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is DinX.
 * 
 * The Initial Developer of the Original Code is Ben Williamson.
 * Portions created by Ben Williamson are Copyright (C) 1999
 * Ben Williamson. All Rights Reserved.
 * 
 * Contributor(s):	Ben Williamson <benw@pobox.com>
 * 
 * Alternatively, the contents of this file may be used under the terms
 * of the GNU General Public license (the  "[GPL] License"), in which case the
 * provisions of [GPL] License are applicable instead of those
 * above.  If you wish to allow use of your version of this file only
 * under the terms of the [GPL] License and not to allow others to use
 * your version of this file under the MPL, indicate your decision by
 * deleting  the provisions above and replace  them with the notice and
 * other provisions required by the [GPL] License.  If you do not delete
 * the provisions above, a recipient may use your version of this file
 * under either the MPL or the [GPL] License.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/sysmacros.h>
#include <sys/ioctl.h>
#include <fcntl.h>

#include <linux/fb.h>

const char *fb_filename = "/dev/fb";

int fb_number;

struct fb_fix_screeninfo fb_fix;
struct fb_var_screeninfo fb_var;
	

int fb_fd, my_fb;

int main(int argc, char **argv)
{
	struct stat st;
	char *env, *buf;
	int x, y;
	
	if (argc > 2) {
		printf("Usage: grab [/dev/fb]\n");
		printf("Writes a PPM image to output stream.\n");
		exit(1);
	}
	env = getenv("FRAMEBUFFER");
	if (env) {
		fb_filename = env;
	}
	if (argc == 2) {
		fb_filename = argv[1];
	}

	if (stat(fb_filename, &st)) {
		perror(fb_filename);
		exit(1);
	}
	if (!S_ISCHR(st.st_mode)) {
		fprintf(stderr, "%s: not a character special device\n",
				fb_filename);
		exit(1);
	}
	if (major(st.st_rdev) != FB_MAJOR) {
		fprintf(stderr, "%s: major number does not match fb.h\n",
				fb_filename);
		exit(1);
	}
	fb_number = minor(st.st_rdev) >> FB_MODES_SHIFT;

	fb_fd = open(fb_filename, O_RDWR);
	if (fb_fd < 0) {
		perror(fb_filename);
		exit(1);
	}

	if (ioctl(fb_fd, FBIOGET_FSCREENINFO, &fb_fix)) goto err_out;
	if (ioctl(fb_fd, FBIOGET_VSCREENINFO, &fb_var)) goto err_out;
	
	buf = malloc(fb_fix.line_length);
	if (!buf) {
		printf("out of memory\n");
		exit(1);
	}
	printf("P6\n%d %d\n255\n", fb_var.xres_virtual, fb_var.yres_virtual);
	fflush(NULL);
	for (y = 0; y < fb_var.yres_virtual; y++) {
		read(fb_fd, buf, fb_fix.line_length);
		for (x = 0; x < fb_var.xres_virtual * 3; x += 3) {
			putchar(buf[x + 2]);
			putchar(buf[x + 1]);
			putchar(buf[x]);
		}
	}
	free(buf);
	close(fb_fd);
	return 0;
err_out:
	perror("grab");
	exit(1);
}

